/*     */ package org.hibernate.cfg;
/*     */ 
/*     */ import java.util.Map;
/*     */ import org.apache.commons.logging.Log;
/*     */ import org.apache.commons.logging.LogFactory;
/*     */ import org.hibernate.AnnotationException;
/*     */ import org.hibernate.AssertionFailure;
/*     */ import org.hibernate.annotations.Index;
/*     */ import org.hibernate.annotations.common.reflection.XClass;
/*     */ import org.hibernate.annotations.common.reflection.XProperty;
/*     */ import org.hibernate.cfg.annotations.Nullability;
/*     */ import org.hibernate.mapping.Join;
/*     */ import org.hibernate.mapping.SimpleValue;
/*     */ import org.hibernate.mapping.Table;
/*     */ import org.hibernate.util.StringHelper;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class Ejb3Column
/*     */ {
/*  26 */   private static final Log log = LogFactory.getLog(Ejb3Column.class);
/*     */   private org.hibernate.mapping.Column mappingColumn;
/*  28 */   private boolean insertable = true;
/*  29 */   private boolean updatable = true;
/*     */   private String secondaryTableName;
/*     */   protected Map<String, Join> joins;
/*     */   protected PropertyHolder propertyHolder;
/*     */   private ExtendedMappings mappings;
/*     */   private boolean isImplicit;
/*     */   public static final int DEFAULT_COLUMN_LENGTH = 255;
/*     */   public String sqlType;
/*  37 */   private int length = 255;
/*     */   private int precision;
/*     */   private int scale;
/*     */   private String logicalColumnName;
/*     */   private String propertyName;
/*     */   private boolean unique;
/*  43 */   private boolean nullable = true;
/*     */   private String formulaString;
/*     */   private org.hibernate.mapping.Formula formula;
/*     */   private Table table;
/*     */   
/*     */   public void setTable(Table table) {
/*  49 */     this.table = table;
/*     */   }
/*     */   
/*     */   public String getLogicalColumnName() {
/*  53 */     return this.logicalColumnName;
/*     */   }
/*     */   
/*     */   public String getSqlType() {
/*  57 */     return this.sqlType;
/*     */   }
/*     */   
/*     */   public int getLength() {
/*  61 */     return this.length;
/*     */   }
/*     */   
/*     */   public int getPrecision() {
/*  65 */     return this.precision;
/*     */   }
/*     */   
/*     */   public int getScale() {
/*  69 */     return this.scale;
/*     */   }
/*     */   
/*     */   public boolean isUnique() {
/*  73 */     return this.unique;
/*     */   }
/*     */   
/*     */   public String getFormulaString() {
/*  77 */     return this.formulaString;
/*     */   }
/*     */   
/*     */   public String getSecondaryTableName() {
/*  81 */     return this.secondaryTableName;
/*     */   }
/*     */   
/*     */   public void setFormula(String formula) {
/*  85 */     this.formulaString = formula;
/*     */   }
/*     */   
/*     */   public boolean isImplicit() {
/*  89 */     return this.isImplicit;
/*     */   }
/*     */   
/*     */   public void setInsertable(boolean insertable) {
/*  93 */     this.insertable = insertable;
/*     */   }
/*     */   
/*     */   public void setUpdatable(boolean updatable) {
/*  97 */     this.updatable = updatable;
/*     */   }
/*     */   
/*     */   protected ExtendedMappings getMappings() {
/* 101 */     return this.mappings;
/*     */   }
/*     */   
/*     */   public void setMappings(ExtendedMappings mappings) {
/* 105 */     this.mappings = mappings;
/*     */   }
/*     */   
/*     */   public void setImplicit(boolean implicit) {
/* 109 */     this.isImplicit = implicit;
/*     */   }
/*     */   
/*     */   public void setSqlType(String sqlType) {
/* 113 */     this.sqlType = sqlType;
/*     */   }
/*     */   
/*     */   public void setLength(int length) {
/* 117 */     this.length = length;
/*     */   }
/*     */   
/*     */   public void setPrecision(int precision) {
/* 121 */     this.precision = precision;
/*     */   }
/*     */   
/*     */   public void setScale(int scale) {
/* 125 */     this.scale = scale;
/*     */   }
/*     */   
/*     */   public void setLogicalColumnName(String logicalColumnName) {
/* 129 */     this.logicalColumnName = logicalColumnName;
/*     */   }
/*     */   
/*     */   public void setPropertyName(String propertyName) {
/* 133 */     this.propertyName = propertyName;
/*     */   }
/*     */   
/*     */   public String getPropertyName() {
/* 137 */     return this.propertyName;
/*     */   }
/*     */   
/*     */   public void setUnique(boolean unique) {
/* 141 */     this.unique = unique;
/*     */   }
/*     */   
/*     */   public boolean isNullable() {
/* 145 */     return this.mappingColumn.isNullable();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public void bind()
/*     */   {
/* 152 */     if (StringHelper.isNotEmpty(this.formulaString)) {
/* 153 */       log.debug("binding formula " + this.formulaString);
/* 154 */       this.formula = new org.hibernate.mapping.Formula();
/* 155 */       this.formula.setFormula(this.formulaString);
/*     */     }
/*     */     else {
/* 158 */       initMappingColumn(this.logicalColumnName, this.propertyName, this.length, this.precision, this.scale, this.nullable, this.sqlType, this.unique, true);
/*     */       
/*     */ 
/* 161 */       log.debug("Binding column " + this.mappingColumn.getName() + " unique " + this.unique);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected void initMappingColumn(String columnName, String propertyName, int length, int precision, int scale, boolean nullable, String sqlType, boolean unique, boolean applyNamingStrategy)
/*     */   {
/* 169 */     this.mappingColumn = new org.hibernate.mapping.Column();
/* 170 */     redefineColumnName(columnName, propertyName, applyNamingStrategy);
/* 171 */     this.mappingColumn.setLength(length);
/* 172 */     if (precision > 0) {
/* 173 */       this.mappingColumn.setPrecision(precision);
/* 174 */       this.mappingColumn.setScale(scale);
/*     */     }
/* 176 */     this.mappingColumn.setNullable(nullable);
/* 177 */     this.mappingColumn.setSqlType(sqlType);
/* 178 */     this.mappingColumn.setUnique(unique);
/*     */   }
/*     */   
/*     */   public boolean isNameDeferred() {
/* 182 */     return (this.mappingColumn == null) || (StringHelper.isEmpty(this.mappingColumn.getName()));
/*     */   }
/*     */   
/*     */   public void redefineColumnName(String columnName, String propertyName, boolean applyNamingStrategy) {
/* 186 */     if (applyNamingStrategy) {
/* 187 */       if (StringHelper.isEmpty(columnName)) {
/* 188 */         if (propertyName != null) {
/* 189 */           this.mappingColumn.setName(this.mappings.getNamingStrategy().propertyToColumnName(propertyName));
/*     */         }
/*     */         
/*     */       }
/*     */       else {
/* 194 */         this.mappingColumn.setName(this.mappings.getNamingStrategy().columnName(columnName));
/*     */       }
/*     */       
/*     */     }
/* 198 */     else if (StringHelper.isNotEmpty(columnName)) this.mappingColumn.setName(columnName);
/*     */   }
/*     */   
/*     */   public String getName()
/*     */   {
/* 203 */     return this.mappingColumn.getName();
/*     */   }
/*     */   
/*     */   public org.hibernate.mapping.Column getMappingColumn() {
/* 207 */     return this.mappingColumn;
/*     */   }
/*     */   
/*     */   public boolean isInsertable() {
/* 211 */     return this.insertable;
/*     */   }
/*     */   
/*     */   public boolean isUpdatable() {
/* 215 */     return this.updatable;
/*     */   }
/*     */   
/*     */   public void setNullable(boolean nullable) {
/* 219 */     if (this.mappingColumn != null) {
/* 220 */       this.mappingColumn.setNullable(nullable);
/*     */     }
/*     */     else {
/* 223 */       this.nullable = nullable;
/*     */     }
/*     */   }
/*     */   
/*     */   public void setJoins(Map<String, Join> joins) {
/* 228 */     this.joins = joins;
/*     */   }
/*     */   
/*     */   public PropertyHolder getPropertyHolder() {
/* 232 */     return this.propertyHolder;
/*     */   }
/*     */   
/*     */   public void setPropertyHolder(PropertyHolder propertyHolder) {
/* 236 */     this.propertyHolder = propertyHolder;
/*     */   }
/*     */   
/*     */   protected void setMappingColumn(org.hibernate.mapping.Column mappingColumn) {
/* 240 */     this.mappingColumn = mappingColumn;
/*     */   }
/*     */   
/*     */   public void linkWithValue(SimpleValue value) {
/* 244 */     if (this.formula != null) {
/* 245 */       value.addFormula(this.formula);
/*     */     }
/*     */     else {
/* 248 */       getMappingColumn().setValue(value);
/* 249 */       value.addColumn(getMappingColumn());
/* 250 */       value.getTable().addColumn(getMappingColumn());
/* 251 */       addColumnBinding(value);
/* 252 */       this.table = value.getTable();
/*     */     }
/*     */   }
/*     */   
/*     */   protected void addColumnBinding(SimpleValue value) {
/* 257 */     String logicalColumnName = this.mappings.getNamingStrategy().logicalColumnName(this.logicalColumnName, this.propertyName);
/*     */     
/* 259 */     this.mappings.addColumnBinding(logicalColumnName, getMappingColumn(), value.getTable());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Table getTable()
/*     */   {
/* 270 */     if (this.table != null) return this.table;
/* 271 */     if (isSecondary()) {
/* 272 */       return getJoin().getTable();
/*     */     }
/*     */     
/* 275 */     return this.propertyHolder.getTable();
/*     */   }
/*     */   
/*     */   public boolean isSecondary()
/*     */   {
/* 280 */     if (this.propertyHolder == null) {
/* 281 */       throw new AssertionFailure("Should not call getTable() on column wo persistent class defined");
/*     */     }
/* 283 */     if (StringHelper.isNotEmpty(this.secondaryTableName)) {
/* 284 */       return true;
/*     */     }
/*     */     
/* 287 */     return false;
/*     */   }
/*     */   
/*     */   public Join getJoin() {
/* 291 */     Join join = (Join)this.joins.get(this.secondaryTableName);
/* 292 */     if (join == null) {
/* 293 */       throw new AnnotationException("Cannot find the expected secondary table: no " + this.secondaryTableName + " available for " + this.propertyHolder.getClassName());
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 299 */     return join;
/*     */   }
/*     */   
/*     */   public void forceNotNull()
/*     */   {
/* 304 */     this.mappingColumn.setNullable(false);
/*     */   }
/*     */   
/*     */   public void setSecondaryTableName(String secondaryTableName) {
/* 308 */     this.secondaryTableName = secondaryTableName;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public static Ejb3Column[] buildColumnFromAnnotation(javax.persistence.Column[] anns, org.hibernate.annotations.Formula formulaAnn, Nullability nullability, PropertyHolder propertyHolder, PropertyData inferredData, Map<String, Join> secondaryTables, ExtendedMappings mappings)
/*     */   {
/*     */     Ejb3Column[] columns;
/*     */     
/*     */     Ejb3Column[] columns;
/*     */     
/* 319 */     if (formulaAnn != null) {
/* 320 */       Ejb3Column formulaColumn = new Ejb3Column();
/* 321 */       formulaColumn.setFormula(formulaAnn.value());
/* 322 */       formulaColumn.setImplicit(false);
/* 323 */       formulaColumn.setMappings(mappings);
/* 324 */       formulaColumn.setPropertyHolder(propertyHolder);
/* 325 */       formulaColumn.bind();
/* 326 */       columns = new Ejb3Column[] { formulaColumn };
/*     */     }
/*     */     else {
/* 329 */       javax.persistence.Column[] actualCols = anns;
/* 330 */       javax.persistence.Column[] overriddenCols = propertyHolder.getOverriddenColumn(StringHelper.qualify(propertyHolder.getPath(), inferredData.getPropertyName()));
/*     */       
/*     */ 
/* 333 */       if (overriddenCols != null)
/*     */       {
/* 335 */         if ((anns != null) && (overriddenCols.length != anns.length)) {
/* 336 */           throw new AnnotationException("AttributeOverride.column() should override all columns for now");
/*     */         }
/* 338 */         actualCols = overriddenCols.length == 0 ? null : overriddenCols;
/* 339 */         log.debug("Column(s) overridden for property " + inferredData.getPropertyName()); }
/*     */       Ejb3Column[] columns;
/* 341 */       if (actualCols == null) {
/* 342 */         columns = buildImplicitColumn(inferredData, secondaryTables, propertyHolder, nullability, mappings);
/*     */       }
/*     */       else {
/* 345 */         int length = actualCols.length;
/* 346 */         columns = new Ejb3Column[length];
/* 347 */         for (int index = 0; index < length; index++) {
/* 348 */           javax.persistence.Column col = actualCols[index];
/* 349 */           String sqlType = col.columnDefinition().equals("") ? null : col.columnDefinition();
/* 350 */           Ejb3Column column = new Ejb3Column();
/* 351 */           column.setImplicit(false);
/* 352 */           column.setSqlType(sqlType);
/* 353 */           column.setLength(col.length());
/* 354 */           column.setPrecision(col.precision());
/* 355 */           column.setScale(col.scale());
/* 356 */           column.setLogicalColumnName(col.name());
/* 357 */           column.setPropertyName(BinderHelper.getRelativePath(propertyHolder, inferredData.getPropertyName()));
/*     */           
/*     */ 
/* 360 */           column.setNullable(col.nullable());
/*     */           
/*     */ 
/* 363 */           column.setUnique(col.unique());
/* 364 */           column.setInsertable(col.insertable());
/* 365 */           column.setUpdatable(col.updatable());
/* 366 */           column.setSecondaryTableName(col.table());
/* 367 */           column.setPropertyHolder(propertyHolder);
/* 368 */           column.setJoins(secondaryTables);
/* 369 */           column.setMappings(mappings);
/* 370 */           column.bind();
/* 371 */           columns[index] = column;
/*     */         }
/*     */       }
/*     */     }
/* 375 */     return columns;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private static Ejb3Column[] buildImplicitColumn(PropertyData inferredData, Map<String, Join> secondaryTables, PropertyHolder propertyHolder, Nullability nullability, ExtendedMappings mappings)
/*     */   {
/* 383 */     Ejb3Column[] columns = new Ejb3Column[1];
/* 384 */     Ejb3Column column = new Ejb3Column();
/* 385 */     column.setImplicit(false);
/*     */     
/* 387 */     if ((nullability != Nullability.FORCED_NULL) && (inferredData.getClassOrElement().isPrimitive()) && (!inferredData.getProperty().isArray()))
/*     */     {
/*     */ 
/* 390 */       column.setNullable(false);
/*     */     }
/* 392 */     column.setLength(255);
/* 393 */     column.setPropertyName(BinderHelper.getRelativePath(propertyHolder, inferredData.getPropertyName()));
/*     */     
/*     */ 
/* 396 */     column.setPropertyHolder(propertyHolder);
/* 397 */     column.setJoins(secondaryTables);
/* 398 */     column.setMappings(mappings);
/* 399 */     column.bind();
/* 400 */     columns[0] = column;
/* 401 */     return columns;
/*     */   }
/*     */   
/*     */   public static void checkPropertyConsistency(Ejb3Column[] columns, String propertyName) {
/* 405 */     int nbrOfColumns = columns.length;
/* 406 */     if (nbrOfColumns > 1) {
/* 407 */       for (int currentIndex = 1; currentIndex < nbrOfColumns; currentIndex++) {
/* 408 */         if (columns[currentIndex].isInsertable() != columns[(currentIndex - 1)].isInsertable()) {
/* 409 */           throw new AnnotationException("Mixing insertable and non insertable columns in a property is not allowed: " + propertyName);
/*     */         }
/*     */         
/*     */ 
/* 413 */         if (columns[currentIndex].isNullable() != columns[(currentIndex - 1)].isNullable()) {
/* 414 */           throw new AnnotationException("Mixing nullable and non nullable columns in a property is not allowed: " + propertyName);
/*     */         }
/*     */         
/*     */ 
/* 418 */         if (columns[currentIndex].isUpdatable() != columns[(currentIndex - 1)].isUpdatable()) {
/* 419 */           throw new AnnotationException("Mixing updatable and non updatable columns in a property is not allowed: " + propertyName);
/*     */         }
/*     */         
/*     */ 
/* 423 */         if (!columns[currentIndex].getTable().equals(columns[(currentIndex - 1)].getTable())) {
/* 424 */           throw new AnnotationException("Mixing different tables in a property is not allowed: " + propertyName);
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public void addIndex(Index index, boolean inSecondPass)
/*     */   {
/* 433 */     if (index == null) return;
/* 434 */     String indexName = index.name();
/* 435 */     addIndex(indexName, inSecondPass);
/*     */   }
/*     */   
/*     */   void addIndex(String indexName, boolean inSecondPass) {
/* 439 */     IndexOrUniqueKeySecondPass secondPass = new IndexOrUniqueKeySecondPass(indexName, this, this.mappings, false);
/* 440 */     if (inSecondPass) {
/* 441 */       secondPass.doSecondPass(this.mappings.getClasses());
/*     */     }
/*     */     else {
/* 444 */       this.mappings.addSecondPass(secondPass);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   void addUniqueKey(String uniqueKeyName, boolean inSecondPass)
/*     */   {
/* 451 */     IndexOrUniqueKeySecondPass secondPass = new IndexOrUniqueKeySecondPass(uniqueKeyName, this, this.mappings, true);
/* 452 */     if (inSecondPass) {
/* 453 */       secondPass.doSecondPass(this.mappings.getClasses());
/*     */     }
/*     */     else {
/* 456 */       this.mappings.addSecondPass(secondPass);
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\Ejb3Column.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */